@@ -1,8 +1,11 @@
+import json
 import logging
+import os
 import random
 import shutil
 import string
 import subprocess
+import tempfile
 from ctypes import c_char, c_ushort, c_uint, c_char_p, c_wchar_p
 from ctypes import windll, Structure, POINTER, sizeof, byref, pointer
 from ctypes.wintypes import HANDLE, DWORD, LPCWSTR, ULONG, LONG
@@ -10,7 +13,7 @@ from _winreg import CreateKeyEx, SetValueEx
 from _winreg import HKEY_CURRENT_USER, HKEY_LOCAL_MACHINE, KEY_ALL_ACCESS
 from _winreg import KEY_SET_VALUE, REG_DWORD, REG_SZ, REG_MULTI_SZ
 
-from settings import HOST_IP, HOST_PORT, RESOLUTION, VMMODE
+import settings as s
 
 
 # http://blogs.technet.com/b/heyscriptingguy/archive/2010/07/07/hey-scripting-guy-how-can-i-change-my-desktop-monitor-resolution-via-windows-powershell.aspx
@@ -125,9 +128,6 @@ REGISTRY = [
 
     # Cloak SystemBios Version.
     (HKEY_LOCAL_MACHINE, 'HARDWARE\\Description\\System', 'VideoBiosVersion', REG_MULTI_SZ, [generate_vga_bios(), generate_vga_bios()]),
-
-    # Install agent.py to be ran on the next startup.
-    (HKEY_LOCAL_MACHINE, 'Software\\Microsoft\\Windows\\CurrentVersion\\Run', 'Agent', REG_SZ, 'C:\\Python27\\Pythonw.exe C:\\agent.py %s %s %s' % (VMMODE, HOST_IP, HOST_PORT)),
 ]
 
 
@@ -195,13 +195,30 @@ class SetupWindows(object):
         agent = open('C:\\vmcloak\\agent.py', 'rb').read()
 
         # Set the desired resolution.
-        width, height = [int(x) for x in RESOLUTION.split('x')]
+        width, height = [int(x) for x in s.resolution.split('x')]
         self.set_resolution(width, height)
 
         # Set registry keys.
         for key, subkey, name, typ, value in REGISTRY:
             self.set_regkey(key, subkey, name, typ, value)
 
+        settings = dict(
+            vmmode=s.vmmode,
+            host_ip=s.host_ip,
+            host_port=s.host_port,
+        )
+
+        fd, agent_path = \
+            tempfile.mkstemp(dir=os.getenv('APPDATA'), suffix='.py')
+        os.close(fd)
+
+        # Install agent.py to be ran on the next startup.
+        value = 'C:\\Python27\\Pythonw.exe "%s" %s' % (
+            agent_path, json.dumps(settings).encode('base64'))
+        self.set_regkey(HKEY_LOCAL_MACHINE,
+                        'Software\\Microsoft\\Windows\\CurrentVersion\\Run',
+                        'Agent', REG_SZ, value)
+
         # Rename registry keys.
         self.rename_regkey(HKEY_LOCAL_MACHINE,
                            'HARDWARE\\ACPI\\DSDT\\VBOX__', random_string())
@@ -213,7 +230,7 @@ class SetupWindows(object):
                            'HARDWARE\\ACPI\\RSDT\\VBOX__', random_string())
 
         # Drop the agent where it'll be executed after restarting the system.
-        open('C:\\agent.py', 'wb').write(agent)
+        open(agent_path, 'wb').write(agent)
         self.log.info('Agent dropped')
 
         # Remove all vmcloak files that are directly related. This does not
@@ -228,7 +245,7 @@ class SetupWindows(object):
         # TODO Instead fork into another process, kill this process, and
         # shutdown from there. Now it's just hanging around.
 
-        if VMMODE == 'bird':
+        if s.vmmode == 'bird':
             # In bird mode we shutdown the Virtual Machine.
             subprocess.Popen(['shutdown', '-s', '-t', '0'])
         else:
